@@ -1,10 +1,18 @@
 package Class::Method::Modifiers;
+{
+  $Class::Method::Modifiers::VERSION = '2.09';
+}
+# git description: v2.08-4-g82f0517
+
+BEGIN {
+  $Class::Method::Modifiers::AUTHORITY = 'cpan:SARTAK';
+}
 use strict;
 use warnings;
 
-our $VERSION = '2.03';
+# work around https://rt.cpan.org/Ticket/Display.html?id=89173
+use base ('Exp'.'orter');
 
-use base 'Exporter';
 our @EXPORT = qw(before after around);
 our @EXPORT_OK = (@EXPORT, qw(fresh install_modifier));
 our %EXPORT_TAGS = (
@@ -12,6 +20,10 @@ our %EXPORT_TAGS = (
     all   => \@EXPORT_OK,
 );
 
+BEGIN {
+  *_HAS_READONLY = $] >= 5.008 ? sub(){1} : sub(){0};
+}
+
 our %MODIFIER_CACHE;
 
 # for backward compatibility
@@ -42,7 +54,7 @@ sub install_modifier {
         };
 
         # this must be the first modifier we're installing
-        if (!exists($MODIFIER_CACHE{$into}{$name}{"orig"})) {
+        if (!exists($cache->{"orig"})) {
             no strict 'refs';
 
             # grab the original method (or undef if the method is inherited)
@@ -74,7 +86,11 @@ sub install_modifier {
         # the Moose equivalent. :)
         if ($type eq 'around') {
             my $method = $cache->{wrapped};
-            $cache->{wrapped} = eval "package $into; sub { \$code->(\$method, \@_); };";
+            my $attrs = _sub_attrs($code);
+            # a bare "sub :lvalue {...}" will be parsed as a label and an
+            # indirect method call. force it to be treated as an expression
+            # using +
+            $cache->{wrapped} = eval "package $into; +sub $attrs { \$code->(\$method, \@_); };";
         }
 
         # install our new method which dispatches the modifiers, but only
@@ -89,8 +105,10 @@ sub install_modifier {
             # to take a reference to it. better a deref than a hash lookup
             my $wrapped = \$cache->{"wrapped"};
 
+            my $attrs = _sub_attrs($cache->{wrapped});
+
             my $generated = "package $into;\n";
-            $generated .= "sub $name {";
+            $generated .= "sub $name $attrs {";
 
             # before is easy, it doesn't affect the return value(s)
             if (@$before) {
@@ -103,12 +121,13 @@ sub install_modifier {
 
             if (@$after) {
                 $generated .= '
-                    my @ret;
+                    my $ret;
                     if (wantarray) {
-                        @ret = $$wrapped->(@_);
+                        $ret = [$$wrapped->(@_)];
+                        '.(_HAS_READONLY ? 'Internals::SvREADONLY(@$ret, 1);' : '').'
                     }
                     elsif (defined wantarray) {
-                        $ret[0] = $$wrapped->(@_);
+                        $ret = \($$wrapped->(@_));
                     }
                     else {
                         $$wrapped->(@_);
@@ -118,8 +137,8 @@ sub install_modifier {
                         $method->(@_);
                     }
 
-                    return wantarray ? @ret : $ret[0];
-                ';
+                    wantarray ? @$ret : $ret ? $$ret : ();
+                '
             }
             else {
                 $generated .= '$$wrapped->(@_);';
@@ -182,13 +201,20 @@ sub _fresh {
             *{"$into\::$name"} = $code;
         }
         else {
-            my $body = 'my $self = shift; $self->$code(@_)';
             no warnings 'closure'; # for 5.8.x
-            eval "package $into; sub $name { $body }";
+            my $attrs = _sub_attrs($code);
+            eval "package $into; sub $name $attrs { \$code->(\@_) }";
         }
     }
 }
 
+sub _sub_attrs {
+    my ($coderef) = @_;
+    local *_sub = $coderef;
+    local $@;
+    (eval 'sub { _sub = 1 }') ? ':lvalue' : '';
+}
+
 sub _is_in_package {
     my ($coderef, $package) = @_;
     require B;
@@ -235,6 +261,8 @@ Class::Method::Modifiers - provides Moose-like method modifiers
 
 =head1 DESCRIPTION
 
+=for stopwords CLOS
+
 Method modifiers are a convenient feature from the CLOS (Common Lisp Object
 System) world.
 
@@ -257,7 +285,8 @@ those methods to flesh out the specifics.
 
 Parent classes need not know about C<Class::Method::Modifiers>. This means you
 should be able to modify methods in I<any> subclass. See
-L<Term::VT102::ZeroBased> for an example of subclassing with CMM.
+L<Term::VT102::ZeroBased> for an example of subclassing with
+C<ClasS::Method::Modifiers>.
 
 In short, C<Class::Method::Modifiers> solves the problem of making sure you
 call C<< $self->SUPER::foo(@_) >>, and provides a cleaner interface for it.
@@ -273,7 +302,7 @@ C<fresh>; see below.
 =head2 before method(s) => sub { ... }
 
 C<before> is called before the method it is modifying. Its return value is
-totally ignored. It receives the same C<@_> as the the method it is modifying
+totally ignored. It receives the same C<@_> as the method it is modifying
 would have received. You can modify the C<@_> the original method will receive
 by changing C<$_[0]> and friends (or by changing anything inside a reference).
 This is a feature!
@@ -281,7 +310,7 @@ This is a feature!
 =head2 after method(s) => sub { ... }
 
 C<after> is called after the method it is modifying. Its return value is
-totally ignored. It receives the same C<@_> as the the method it is modifying
+totally ignored. It receives the same C<@_> as the method it is modifying
 received, mostly. The original method can modify C<@_> (such as by changing
 C<$_[0]> or references) and C<after> will see the modified version. If you
 don't like this behavior, specify both a C<before> and C<after>, and copy the
@@ -386,17 +415,32 @@ by other code. C<Class::Method::Modifiers> provides a way of
 overriding/augmenting methods safely, and the parent class need not know about
 it.
 
+=head2 :lvalue METHODS
+
+When adding C<before> or C<after> modifiers, the wrapper method will be
+an lvalue method if the wrapped sub is, and assigning to the method
+will propagate to the wrapped method as expected.  For C<around>
+modifiers, it is the modifier sub that determines if the wrapper
+method is an lvalue method.
+
 =head1 CAVEATS
 
 It is erroneous to modify a method that doesn't exist in your class's
 inheritance hierarchy. If this occurs, an exception will be thrown when
 the modifier is defined.
 
-It doesn't yet play well with C<caller>. There are some todo tests for this.
+It doesn't yet play well with C<caller>. There are some C<TODO> tests for this.
 Don't get your hopes up though!
 
+Applying modifiers to array lvalue methods is not fully supported. Attempting
+to assign to an array lvalue method that has an C<after> modifier applied will
+result in an error.  Array lvalue methods are not well supported by perl in
+general, and should be avoided.
+
 =head1 VERSION
 
+=for stopwords reimplementation
+
 This module was bumped to 1.00 following a complete reimplementation, to
 indicate breaking backwards compatibility. The "guard" modifier was removed,
 and the internals are completely different.
@@ -419,6 +463,8 @@ Shawn M Moore, C<sartak@gmail.com>
 
 =head1 ACKNOWLEDGEMENTS
 
+=for stopwords Stevan
+
 Thanks to Stevan Little for L<Moose>, I would never have known about
 method modifiers otherwise.
 
